#include <rccl.h>
#include "manex.PA10.h"
#include <fastmath.h>
#include <unistd.h> /*sleep*/


#define BOXSIZE         100.0
#ifdef Vxw_TARGET
#endif

void moveSet(MANIP *mnp, POS_PTR *p, int size)
{
	int k;

	for (k=0; k<size; k++) {
		move (mnp, p[k]);
	}
}

int wheelFxn(TRSF_PTR t, int arg)
{
	ROTATE_CTRL_BLK *rcb;
	static float alpha = 0.0;

	if ((rcb = (ROTATE_CTRL_BLK *) arg) == NULL) {
		rciAbort (0, "wheelFxn(): can't find argument object\n");
		return -1;
	}
#ifdef Vxw_TARGET
	if (!rcb->wheelFxnInit) {
		alpha = 0.0;
		rcb->wheelFxnInit = 1;
	}
#endif
	alpha += rcb->wheelRPS * rcclGetInterval() / 1000.0;

	t->p.x = rcb->wheelRadius * COS(alpha * PIT2);
	t->p.y = rcb->wheelRadius * SIN(alpha * PIT2);

	return 0;
}

int tableFxn(TRSF_PTR t, int arg, MANIP *mnp)
{
	ROTATE_CTRL_BLK *rcb;
	static float alpha = 0.0;
	static int done = 0;

	if ((rcb = (ROTATE_CTRL_BLK *) arg) == NULL)
	{ rciAbort (0, "tableFxn(): can't find argument object\n");
		return -1;
	}
#ifdef Vxw_TARGET
	if (!rcb->tableFxnInit) {
		alpha = 0.0;
		done = 0;
		rcb->tableFxnInit = 1;
	}
#endif
	alpha += 360.0 * rcb->tableRPS * rcclGetInterval() / 1000.0;

	rotToTrsf (t, zunit, alpha);

	if (alpha >= 90.0 && !done) {
		stopCurrentMotion (mnp, 1);
		done = 1;
	}

	return 0;
}

int main(){
	JNTS rcclpark1;
	JNTS rcclpark2;
	TRSF_PTR table0, table, p, wheel, e, b, c[4];
	POS_PTR p0, pt, corners[4], start;
	MANIP *mnp1, *mnp2;
	ROTATE_CTRL_BLK *rcb;
	char *robotName1 = "left";
	char *robotName2 = "right";

	int tableFxn();
	int wheelFxn();
	int mid1, k, ia, iv;

	static float accelValues[] = { F_DEFAULT, 2000.0, 20.0 };
	static char *accelDesc[] = { "default", "high", "low" };
	static float velValues[] = { F_DEFAULT, 300.0, 30.0 };
	static char *velDesc[] = { "default", "fast", "slow" };

	rcclSetOptions (RCCL_ERROR_EXIT);
#if 1
	rcclSetOptions (RCCL_SIMULATE);
	rcclSetInterval(50);
#else
	robotName1 = getDefaultRobot();
	robotName2 = getDefaultRobot();
#endif

	if (!getRobotPosition (rcclpark1.v, "rcclpark", robotName1)) {
		printf ("position ’rcclpark’ not defined for robot\n");
		return(-1);
	}

	if (!getRobotPosition (rcclpark2.v, "rcclpark", robotName2)) {
		printf ("position ’rcclpark’ not defined for robot\n");
		return(-1);
	}

	e = allocTransXyz (0.0, 0.0, TOOLZ);
	b = allocTransRot (B_X, B_Y, B_Z, xunit, 180.0);
	start = makePosition (NULL, T6, e, EQ, b, TL, e, NULL);

	c[0] = allocTransXyz ( BOXSIZE/2, 0.0, -BOXSIZE/2);
	c[1] = allocTransXyz (-BOXSIZE/2, 0.0, -BOXSIZE/2);
	c[2] = allocTransXyz (-BOXSIZE/2, 0.0,  BOXSIZE/2);
	c[3] = allocTransXyz ( BOXSIZE/2, 0.0,  BOXSIZE/2);
	for (k=0; k<4; k++) {
		corners[k] = makePosition (NULL, T6, e, EQ, b, c[k], TL, e,NULL);
	}

	p = allocTransRot (P_X, P_Y, P_Z, xunit, 180.0);
	table0 = allocTransRot (0.0, 0.0, 0.0, zunit, -45.0);
	wheel = allocTrans ();
	table = allocTrans ();

	rcb = (ROTATE_CTRL_BLK *)malloc(sizeof(ROTATE_CTRL_BLK));
	rcb->wheelRadius = 40.0;
	rcb->wheelRPS = 0.40;
	rcb->tableRPS = 0.0; //0.02

	p0 = makePosition ("p0", T6, EQ, table0, p, wheel, TL, T6);
	pt = makePosition ("pt", T6, EQ, table0, table, p, wheel, TL, T6);

	printf("rcclCreate(%s)...\n", robotName1);
	mnp1 = rcclCreate (robotName1);
	sleep(1);
	printf("rcclCreate(%s)...\n", robotName2);
	mnp2 = rcclCreate (robotName2);

	printf("rcclStart()...\n");
	rcclStart();

	transEval (wheel, wheelFxn, /*speed=*/rcb->wheelRPS);

	movej (mnp1, &rcclpark1);
	move (mnp2, start);
	stop (mnp2, 1000.0);
	waitForCompleted (mnp2);

	setMod (mnp1, 'c');
	setMod (mnp2, 'c');
	move (mnp1, p0);

	for (ia=0; ia<ARRAY_SIZE(accelValues); ia++)
	{ for (iv=0; iv<ARRAY_SIZE(velValues); iv++)
		{ float vel, accel;

			setCartAccel (mnp2, accelValues[ia], F_UNDEF);
			setCartVel (mnp2, velValues[iv], F_UNDEF);
			getCartAccel (mnp2, &accel, (float*)NULL);
			getCartVel (mnp2, &vel, (float*)NULL);
			p0 = makePosition ("p0", T6, EQ, table0, p, wheel, TL, T6);
			pt = makePosition ("pt", T6, EQ, table0, table, p, wheel, TL, T6);
			transEval (wheel, wheelFxn, /*speed=*/rcb->wheelRPS);
			transMotionEval (table, tableFxn, rcb->wheelRPS, mnp1);
			move (mnp1, pt);
			printf ("%s vel (%g), %s acceleration (%g):\n",
					velDesc[iv], vel, accelDesc[ia], accel);

			moveSet (mnp2, corners, ARRAY_SIZE(corners));
			waitForCompleted (mnp2);
		}
	}


	mid1 = stop (mnp1, F_UNDEF);

	transMotionEval (table, tableFxn, rcb->wheelRPS, mnp1);

	setTime (mnp1, F_DEFAULT, F_UNDEF);

	move (mnp1, pt);

	movej (mnp1, &rcclpark1);

	stop (mnp1, 1000.0);

	waitFor (motionStatus(mid1) == M_RUNNING);

	delay (10000.0);
	stopMotion (mid1, 0);

	setMod (mnp2, 'j');
	move (mnp2, start);
	stop (mnp2, 1000.0);
	waitForCompleted (mnp2);

	waitForCompleted (mnp1);

	rcclRelease (1);
	return 0;
}
