#ifdef CONFIGFILE
#include CONFIGFILE
#endif
#include <rccl.h>
#include <puma_kynvar.h>
#include <errorCodes.h>

#include "impedance.h"

#define SENSOR_Z_OFFSET         105.0

#define NEWTONS_PER_POUND       4.448
#define LB_PER_UF               (1.0/40.0)
#define NEWTONS_PER_UF          (4.448/40.0)
#define MM_PER_INCH             25.4

#define SGN(x)  ((x) >= 0 ? 1 : -1)

calcForce(arg, mnp)
int arg;
MANIP *mnp;
{
	COMPLY_CTRL_BLK *cbk;
	FORCE force1;
	FORCE force2;
	char *fxnName = "calcForce()";
	/*10 */
	if ((cbk = (COMPLY_CTRL_BLK *) getMemByAddr((void *)arg)) == NULL) {
		rciAbort(EFatal, "%s -- can't find control block\n", fxnName);
		return;
	}

	cbk->valid = 0;
	if (cbk->init) {
		SET_FORCE_READ(mnp->rbt, 1);	/*12 */
		cbk->init = NO;
		cbk->valid = 0;
	} else {
		if (mnp->how->forceOK) {	/*13 */
			TRSF stoc;

			force1.f.x = mnp->how->forceInput[0];
			force1.f.y = mnp->how->forceInput[1];
			force1.f.z = mnp->how->forceInput[2];

			force1.m.x = mnp->how->forceInput[3];
			force1.m.y = mnp->how->forceInput[4];
			force1.m.z = mnp->how->forceInput[5];

			if (cbk->resetBias) {
				cbk->bias = force1;
				cbk->resetBias = NO;
			} else {
				subtractVf((float *)&force1, (float *)&force1,
					   (float *)&cbk->bias, 6);

				stoc = *mnp->tool;
				stoc.p.z -= SENSOR_Z_OFFSET;	/*14 */
				transForce(&force2, &force1, &stoc);
				accessMem(cbk, &force2, &cbk->force,
					  6 * sizeof(float));
				cbk->valid = 1;	/*15 */
			}
		}
	}
}

double calcVel(params, fval, fbias, err)
CtrlParams *params;
double fval, fbias, err;
{
	double vel;

	if (ABS(fval) > params->deadband) {
		fval -= SGN(fval) * params->deadband;
	} else {
		fval = 0;
	}
	vel = (fval - fbias + params->stiffness * err) * params->invDamping;
	return (vel);
}

complyFxn(t, arg, mnp)
TRSF *t;
int arg;
MANIP *mnp;
{
	char *fxnName = "complyFxn()";

	COMPLY_CTRL_BLK *cbk;
	FORCE complyValues;
	unsigned long complyMask;
	TRSF accomTrsf;
	TRSF Temp;
	TRSF Err;
	DIFF err;
	DIFF vel;

	if ((cbk = (COMPLY_CTRL_BLK *) getMemByAddr((void *)arg)) == NULL) {
		rciAbort(EFatal, "%s -- can't find control block\n", fxnName);
		return;
	}

	if (!cbk->valid) {
		if (cbk->badDataCount++ > 4) {	/*17 */
			rciAbort(0,
				 "bad force data for 4 consecutive cycles\n");
		}
		return;
	}
	cbk->badDataCount = 0;

	zeroVf((float *)&vel, 6);

	/* Calculate error matrix */
	multTrsf(&Temp, mnp->t6, mnp->tool);
	multLiTrsf(&Temp, &Temp, mnp->t6o);
	multTrsf(&Temp, &Temp, mnp->tool);
	multTrsf(&Err, &Temp, getTransByAddr(cbk->comply));

	/* convert this to a DIFF structure */
	trsfToDiff(&err, &Err);

	getActiveComply(mnp, &complyMask, &complyValues);	/*18 */

	if (complyMask & ALONG_X) {
		vel.t.x = calcVel(&cbk->tpar, cbk->force.f.x,
				  complyValues.f.x, err.t.x);
	}
	if (complyMask & ALONG_Y) {
		vel.t.y = calcVel(&cbk->tpar, cbk->force.f.y,
				  complyValues.f.y, err.t.y);
	}
	if (complyMask & ALONG_Z) {
		vel.t.z = calcVel(&cbk->tpar, cbk->force.f.z,
				  complyValues.f.z, err.t.z);
	}
	if (complyMask & ABOUT_X) {
		vel.r.x = calcVel(&cbk->tpar, cbk->force.m.x,
				  complyValues.m.x, err.r.x);
	}
	if (complyMask & ABOUT_Y) {
		vel.r.y = calcVel(&cbk->tpar, cbk->force.m.y,
				  complyValues.m.y, err.r.y);
	}
	if (complyMask & ABOUT_Z) {
		vel.r.z = calcVel(&cbk->tpar, cbk->force.m.z,
				  complyValues.m.z, err.r.z);
	}
	scaleVf((float *)&vel, rcclGetInterval() / 1000.0, (float *)&vel, 6);
	diffToTrsf(&accomTrsf, &vel);
	multLiTrsf(t, &accomTrsf, t);	/*21 */
}
